use strict;
use Test::More 'no_plan';
use Data::Dumper;
use SQL::Interpolate qw(:all);
require 't/lib.pl';

my $interp = new SQL::Interpolate();
$interp->relations({
    messageset => {name => qr/([A-Z])/, key => ['msid']},
    message    => {name => qr/([m-p])/, key => ['mid']},
    messageset_message => {
        name => qr/([A-Z])([m-p])/, key => ['msid', 'mid']},
    messageset_messageset => {
        name => qr/([A-Z])([A-Z])/, key => ['msid_1', 'msid_2']}
});

# AB and BC
&filter_sql_test($interp,
    'SELECT * FROM R(AB), R(BC) WHERE LINK(AB,BC) A B C',
    'SELECT * FROM messageset_messageset as AB, messageset_messageset as BC WHERE AB.msid_2 = BC.msid_1 AB.msid_1 BC.msid_1 BC.msid_2', 't1');
&filter_sql_test($interp,
    'SELECT * FROM R(BA), R(BC) WHERE LINK(BA,BC) A B C',
    'SELECT * FROM messageset_messageset as BA, messageset_messageset as BC WHERE BA.msid_1 = BC.msid_1 BA.msid_2 BC.msid_1 BC.msid_2', 't2');
&filter_sql_test($interp,
    'SELECT * FROM R(AB), R(CB) WHERE LINK(AB,CB) A B C',
    'SELECT * FROM messageset_messageset as AB, messageset_messageset as CB WHERE AB.msid_2 = CB.msid_2 AB.msid_1 CB.msid_2 CB.msid_1', 't3');
&filter_sql_test($interp,
    'SELECT * FROM R(AB), R(BC) WHERE LINK(BC,AB) A B C',
    'SELECT * FROM messageset_messageset as AB, messageset_messageset as BC WHERE BC.msid_1 = AB.msid_2 AB.msid_1 BC.msid_1 BC.msid_2', 't1b');

# AB and Bm
&filter_sql_test($interp,
    'SELECT * FROM R(AB), R(Bm) WHERE LINK(AB,Bm) A B m',
    'SELECT * FROM messageset_messageset as AB, messageset_message as Bm WHERE AB.msid_2 = Bm.msid AB.msid_1 Bm.msid Bm.mid', 't1');
&filter_sql_test($interp,
    'SELECT * FROM R(BA), R(Bm) WHERE LINK(BA,Bm) A B m',
    'SELECT * FROM messageset_messageset as BA, messageset_message as Bm WHERE BA.msid_1 = Bm.msid BA.msid_2 Bm.msid Bm.mid', 't1');

# AB and A
&filter_sql_test($interp,
    'SELECT * FROM R(AB), R(A) WHERE LINK(AB,A) A B',
    'SELECT * FROM messageset_messageset as AB, messageset as A WHERE AB.msid_1 = A.msid A.msid AB.msid_2', 't1');
&filter_sql_test($interp,
    'SELECT * FROM R(BA), R(A) WHERE LINK(BA,A) A B',
    'SELECT * FROM messageset_messageset as BA, messageset as A WHERE BA.msid_2 = A.msid A.msid BA.msid_1', 't1');
&filter_sql_test($interp,
    'SELECT * FROM R(AB), R(A) WHERE LINK(A,AB) A B',
    'SELECT * FROM messageset_messageset as AB, messageset as A WHERE A.msid = AB.msid_1 A.msid AB.msid_2', 't1');

# Am and m
&filter_sql_test($interp,
    'SELECT * FROM R(Am), R(m) WHERE LINK(Am,m)',
    'SELECT * FROM messageset_message as Am, message as m WHERE Am.mid = m.mid', 't1');
&filter_sql_test($interp,
    'SELECT * FROM R(Am), R(m) WHERE LINK(m,Am)',
    'SELECT * FROM messageset_message as Am, message as m WHERE m.mid = Am.mid', 't1');

# Am and A
&filter_sql_test($interp,
    'SELECT * FROM R(Am), R(A) WHERE LINK(Am,A)',
    'SELECT * FROM messageset_message as Am, messageset as A WHERE Am.msid = A.msid', 't1');
&filter_sql_test($interp,
    'SELECT * FROM R(Am), R(A) WHERE LINK(A,Am)',
    'SELECT * FROM messageset_message as Am, messageset as A WHERE A.msid = Am.msid', 't1');

# AB and BC and CA
&filter_sql_test($interp,
    'SELECT * FROM R(AB), R(BC), R(CA) WHERE LINK(AB,BC,CA)',
    'SELECT * FROM messageset_messageset as AB, messageset_messageset as BC, messageset_messageset as CA WHERE (AB.msid_2 = BC.msid_1 AND BC.msid_2 = CA.msid_1 AND AB.msid_1 = CA.msid_2)', 't1');

# AB and BC and Cm and m
&filter_sql_test($interp,
    'SELECT * FROM R(AB), R(BC), R(Cm), R(m) WHERE LINK(AB,BC,Cm,m)',
    'SELECT * FROM messageset_messageset as AB, messageset_messageset as BC, messageset_message as Cm, message as m WHERE (AB.msid_2 = BC.msid_1 AND BC.msid_2 = Cm.msid AND Cm.mid = m.mid)', 't1');

# Am and Bm and Cm
&filter_sql_test($interp,
    'SELECT * FROM R(Am), R(Bm), R(Cm) WHERE LINK(Am,Bm,Cm) AND A B C m',
    'SELECT * FROM messageset_message as Am, messageset_message as Bm, messageset_message as Cm WHERE (Am.mid = Bm.mid AND Bm.mid = Cm.mid) AND Am.msid Bm.msid Cm.msid Cm.mid', 't1');


# IMPROVE--handle cases like this
$interp->relations({
    messageset => {name => qr/([A-Z])/, key => [msid => undef]},
    message    => {name => qr/([m-p])/, key => [mid  => undef]},
    user       => {name => qr/([u-z])/, key => [uid  => undef]},
    messageset_message => {name => qr/([A-Z])([m-p])([u-z])/, key => [
        msid   => 'messageset',
        mid    => 'message',
        uid    => 'user'
    ]},
    messageset_messageset => {name => qr/([A-Z])([A-Z])/, key => [
        msid_1 => 'messageset',
        msid_2 => 'messageset'
    ]},
});
# SELECT * FROM R(Amu), R(m), R(u) WHERE LINK(Amu,m,u)

#IMPROVE--handle this (auto-LINK AB and BC)
# SELECT * FROM R(AB,BC)
# SELECT * FROM R(AB LEFT JOIN BC)


$interp = new SQL::Interpolate();
$interp->relations({
    sales_order => {name => qr/([S-T])/, key => ['so_nbr']},
    part => {name => qr/([p-r])/, key => ['part_nbr']},
    sales_order_line => {
        name => qr/([S-T])([p-r])/, key => ['so_nbr', 'part_nbr']}
});

&filter_sql_test($interp,
    "SELECT * FROM R(S), R(Sp), R(p) WHERE LINK(S,Sp,p) AND S = 123",
    "SELECT * FROM sales_order as S, sales_order_line as Sp, part as p WHERE (S.so_nbr = Sp.so_nbr AND Sp.part_nbr = p.part_nbr) AND Sp.so_nbr = 123", 't');


sub filter_sql_test
{
    my($interp, $input, $expect, $name) = @_;
    is_deeply($interp->filter_sql($input), $expect, $name);
}